@@ -5,6 +5,8 @@ Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
 #include "ColumnsPanel.h"
 
 #include <assert.h>
@@ -19,15 +21,15 @@ in the source distribution for its full text.
 #include "Object.h"
 #include "Process.h"
 #include "ProvideCurses.h"
+#include "RowField.h"
 #include "XUtils.h"
 
 
 static const char* const ColumnsFunctions[] = {"      ", "      ", "      ", "      ", "      ", "      ", "MoveUp", "MoveDn", "Remove", "Done  ", NULL};
 
 static void ColumnsPanel_delete(Object* object) {
-   Panel* super = (Panel*) object;
    ColumnsPanel* this = (ColumnsPanel*) object;
-   Panel_done(super);
+   Panel_done(&this->super);
    free(this);
 }
 
@@ -38,14 +40,13 @@ static HandlerResult ColumnsPanel_eventHandler(Panel* super, int ch) {
    HandlerResult result = IGNORED;
    int size = Panel_size(super);
 
-   switch(ch) {
+   switch (ch) {
       case 0x0a:
       case 0x0d:
       case KEY_ENTER:
       case KEY_MOUSE:
       case KEY_RECLICK:
-      {
-         if (selected < size - 1) {
+         if (selected < size) {
             this->moving = !(this->moving);
             Panel_setSelectionColor(super, this->moving ? PANEL_SELECTION_FOLLOW : PANEL_SELECTION_FOCUS);
             ListItem* selectedItem = (ListItem*) Panel_getSelected(super);
@@ -54,59 +55,46 @@ static HandlerResult ColumnsPanel_eventHandler(Panel* super, int ch) {
             result = HANDLED;
          }
          break;
-      }
       case KEY_UP:
-      {
-         if (!this->moving) {
+         if (!this->moving)
             break;
-         }
-      }
          /* else fallthrough */
       case KEY_F(7):
       case '[':
       case '-':
-      {
-         if (selected < size - 1)
+         if (selected < size)
             Panel_moveSelectedUp(super);
          result = HANDLED;
          break;
-      }
       case KEY_DOWN:
-      {
-         if (!this->moving) {
+         if (!this->moving)
             break;
-         }
-      }
          /* else fallthrough */
       case KEY_F(8):
       case ']':
       case '+':
-      {
-         if (selected < size - 2)
+         if (selected < size - 1)
             Panel_moveSelectedDown(super);
          result = HANDLED;
          break;
-      }
       case KEY_F(9):
       case KEY_DC:
-      {
-         if (selected < size - 1) {
+      case KEY_DEL_MAC:
+         if (size > 1 && selected < size)
             Panel_remove(super, selected);
-         }
          result = HANDLED;
          break;
-      }
       default:
-      {
          if (0 < ch && ch < 255 && isgraph((unsigned char)ch))
             result = Panel_selectByTyping(super, ch);
          if (result == BREAK_LOOP)
             result = IGNORED;
          break;
-      }
    }
+
    if (result == HANDLED)
       ColumnsPanel_update(super);
+
    return result;
 }
 
@@ -128,9 +116,8 @@ static void ColumnsPanel_add(Panel* super, unsigned int key, Hashtable* columns)
       if (!column) {
          name = NULL;
       } else {
-         name = column->caption ? column->caption : column->heading;
-         if (!name)
-            name = column->name; /* name is a mandatory field */
+         /* heading preferred here but name is always available */
+         name = column->heading ? column->heading : column->name;
       }
    }
    if (name == NULL)
@@ -139,16 +126,17 @@ static void ColumnsPanel_add(Panel* super, unsigned int key, Hashtable* columns)
 }
 
 void ColumnsPanel_fill(ColumnsPanel* this, ScreenSettings* ss, Hashtable* columns) {
-   Panel* super = (Panel*) this;
+   Panel* super = &this->super;
    Panel_prune(super);
-   for (const ProcessField* fields = ss->fields; *fields; fields++)
+   for (const RowField* fields = ss->fields; *fields; fields++)
       ColumnsPanel_add(super, *fields, columns);
    this->ss = ss;
 }
 
 ColumnsPanel* ColumnsPanel_new(ScreenSettings* ss, Hashtable* columns, bool* changed) {
    ColumnsPanel* this = AllocThis(ColumnsPanel);
-   Panel* super = (Panel*) this;
+   Panel* super = &this->super;
+
    FunctionBar* fuBar = FunctionBar_new(ColumnsFunctions, NULL, NULL);
    Panel_init(super, 1, 1, 1, 1, Class(ListItem), true, fuBar);
 
