@@ -1,18 +1,22 @@
 #ifndef HEADER_ProvideCurses
 #define HEADER_ProvideCurses
 /*
-htop - RichString.h
+htop - ProvideCurses.h
 (C) 2004,2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
 
-#include "config.h"
+// This header is also used in tests by configure, thus conditionally
+// including "config.h".
+#if defined(HAVE_CONFIG_H)
+#include "config.h" // IWYU pragma: keep
+#endif
 
 // IWYU pragma: begin_exports
 
-#ifdef HAVE_NCURSESW_CURSES_H
+#if defined(HAVE_NCURSESW_CURSES_H)
 #include <ncursesw/curses.h>
 #elif defined(HAVE_NCURSES_NCURSES_H)
 #include <ncurses/ncurses.h>
