@@ -16,13 +16,14 @@ in the source distribution for its full text.
 
 #include "CRT.h"
 #include "Process.h"
+#include "darwin/DarwinMachine.h"
 #include "darwin/Platform.h"
 
 
 const ProcessFieldData Process_fields[LAST_PROCESSFIELD] = {
    [0] = { .name = "", .title = NULL, .description = NULL, .flags = 0, },
    [PID] = { .name = "PID", .title = "PID", .description = "Process/thread ID", .flags = 0, .pidColumn = true, },
-   [COMM] = { .name = "Command", .title = "Command ", .description = "Command line", .flags = 0, },
+   [COMM] = { .name = "Command", .title = "Command ", .description = "Command line (insert as last column only)", .flags = 0, },
    [STATE] = { .name = "STATE", .title = "S ", .description = "Process state (S sleeping, R running, D disk, Z zombie, T traced, W paging)", .flags = 0, },
    [PPID] = { .name = "PPID", .title = "PPID", .description = "Parent process ID", .flags = 0, .pidColumn = true, },
    [PGRP] = { .name = "PGRP", .title = "PGRP", .description = "Process group ID", .flags = 0, .pidColumn = true, },
@@ -39,7 +40,7 @@ const ProcessFieldData Process_fields[LAST_PROCESSFIELD] = {
    [M_VIRT] = { .name = "M_VIRT", .title = " VIRT ", .description = "Total program size in virtual memory", .flags = 0, .defaultSortDesc = true, },
    [M_RESIDENT] = { .name = "M_RESIDENT", .title = "  RES ", .description = "Resident set size, size of the text and data sections, plus stack usage", .flags = 0, .defaultSortDesc = true, },
    [ST_UID] = { .name = "ST_UID", .title = "UID", .description = "User ID of the process owner", .flags = 0, },
-   [PERCENT_CPU] = { .name = "PERCENT_CPU", .title = " CPU%", .description = "Percentage of the CPU time the process used in the last sampling", .flags = 0, .defaultSortDesc = true, .autoWidth = true, },
+   [PERCENT_CPU] = { .name = "PERCENT_CPU", .title = " CPU%", .description = "Percentage of the CPU time the process used in the last sampling", .flags = 0, .defaultSortDesc = true, .autoWidth = true, .autoTitleRightAlign = true, },
    [PERCENT_NORM_CPU] = { .name = "PERCENT_NORM_CPU", .title = "NCPU%", .description = "Normalized percentage of the CPU time the process used in the last sampling (normalized by cpu count)", .flags = 0, .defaultSortDesc = true, .autoWidth = true, },
    [PERCENT_MEM] = { .name = "PERCENT_MEM", .title = "MEM% ", .description = "Percentage of the memory the process is using, based on resident memory size", .flags = 0, .defaultSortDesc = true, },
    [USER] = { .name = "USER", .title = "USER       ", .description = "Username of the process owner (or user ID if name cannot be determined)", .flags = 0, },
@@ -51,17 +52,18 @@ const ProcessFieldData Process_fields[LAST_PROCESSFIELD] = {
    [TRANSLATED] = { .name = "TRANSLATED", .title = "T ", .description = "Translation info (T translated, N native)", .flags = 0, },
 };
 
-Process* DarwinProcess_new(const Settings* settings) {
+Process* DarwinProcess_new(const Machine* host) {
    DarwinProcess* this = xCalloc(1, sizeof(DarwinProcess));
    Object_setClass(this, Class(DarwinProcess));
-   Process_init(&this->super, settings);
+   Process_init(&this->super, host);
 
    this->utime = 0;
    this->stime = 0;
    this->taskAccess = true;
    this->translated = false;
+   this->super.state = UNKNOWN;
 
-   return &this->super;
+   return (Process*)this;
 }
 
 void Process_delete(Object* cast) {
@@ -71,18 +73,21 @@ void Process_delete(Object* cast) {
    free(this);
 }
 
-static void DarwinProcess_writeField(const Process* this, RichString* str, ProcessField field) {
-   const DarwinProcess* dp = (const DarwinProcess*) this;
+static void DarwinProcess_rowWriteField(const Row* super, RichString* str, ProcessField field) {
+   const DarwinProcess* dp = (const DarwinProcess*) super;
+
    char buffer[256]; buffer[255] = '\0';
    int attr = CRT_colors[DEFAULT_COLOR];
-   int n = sizeof(buffer) - 1;
+   size_t n = sizeof(buffer) - 1;
+
    switch (field) {
    // add Platform-specific fields here
    case TRANSLATED: xSnprintf(buffer, n, "%c ", dp->translated ? 'T' : 'N'); break;
    default:
-      Process_writeField(this, str, field);
+      Process_writeField(&dp->super, str, field);
       return;
    }
+
    RichString_appendWide(str, attr, buffer);
 }
 
@@ -270,13 +275,6 @@ ERROR_A:
    Process_updateCmdline(proc, k->kp_proc.p_comm, 0, strlen(k->kp_proc.p_comm));
 }
 
-// Converts nanoseconds to hundredths of a second (centiseconds) as needed by the "time" field of the Process struct.
-static long long int nanosecondsToCentiseconds(uint64_t nanoseconds) {
-   const uint64_t centiseconds_per_second = 100;
-   const uint64_t nanoseconds_per_second = 1e9;
-   return nanoseconds / nanoseconds_per_second * centiseconds_per_second;
-}
-
 static char* DarwinProcess_getDevname(dev_t dev) {
    if (dev == NODEV) {
       return NULL;
@@ -291,13 +289,14 @@ static char* DarwinProcess_getDevname(dev_t dev) {
 
 void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps, bool exists) {
    DarwinProcess* dp = (DarwinProcess*)proc;
+   const Settings* settings = proc->super.host->settings;
 
    const struct extern_proc* ep = &ps->kp_proc;
 
    /* UNSET HERE :
     *
     * processor
-    * user (set at ProcessList level)
+    * user (set at ProcessTable level)
     * nlwp
     * percent_cpu
     * percent_mem
@@ -310,12 +309,12 @@ void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps,
    /* First, the "immutable" parts */
    if (!exists) {
       /* Set the PID/PGID/etc. */
-      proc->pid = ep->p_pid;
-      proc->ppid = ps->kp_eproc.e_ppid;
+      Process_setPid(proc, ep->p_pid);
+      Process_setThreadGroup(proc, ep->p_pid);
+      Process_setParent(proc, ps->kp_eproc.e_ppid);
       proc->pgrp = ps->kp_eproc.e_pgid;
       proc->session = 0; /* TODO Get the session id */
       proc->tpgid = ps->kp_eproc.e_tpgid;
-      proc->tgid = proc->pid;
       proc->isKernelThread = false;
       proc->isUserlandThread = false;
       dp->translated = ps->kp_proc.p_flag & P_TRANSLATED;
@@ -328,7 +327,7 @@ void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps,
       DarwinProcess_updateExe(ep->p_pid, proc);
       DarwinProcess_updateCmdLine(ps, proc);
 
-      if (proc->settings->ss->flags & PROCESS_FLAG_CWD) {
+      if (settings->ss->flags & PROCESS_FLAG_CWD) {
          DarwinProcess_updateCwd(ep->p_pid, proc);
       }
    }
@@ -341,7 +340,7 @@ void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps,
        * To mitigate this we only fetch TTY information if the TTY
        * field is enabled in the settings.
        */
-      if (proc->settings->ss->flags & PROCESS_FLAG_TTY) {
+      if (settings->ss->flags & PROCESS_FLAG_TTY) {
          proc->tty_name = DarwinProcess_getDevname(proc->tty_nr);
          if (!proc->tty_name) {
             /* devname failed: prevent us from calling it again */
@@ -357,44 +356,48 @@ void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps,
    proc->state = (ep->p_stat == SZOMB) ? ZOMBIE : UNKNOWN;
 
    /* Make sure the updated flag is set */
-   proc->updated = true;
+   proc->super.updated = true;
 }
 
-void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessList* dpl, double timeIntervalNS) {
+void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessTable* dpt, double timeIntervalNS) {
    struct proc_taskinfo pti;
 
-   if (sizeof(pti) == proc_pidinfo(proc->super.pid, PROC_PIDTASKINFO, 0, &pti, sizeof(pti))) {
-      uint64_t total_existing_time_ns = proc->stime + proc->utime;
-
-      uint64_t user_time_ns = Platform_machTicksToNanoseconds(pti.pti_total_user);
-      uint64_t system_time_ns = Platform_machTicksToNanoseconds(pti.pti_total_system);
-
-      uint64_t total_current_time_ns = user_time_ns + system_time_ns;
-
-      if (total_existing_time_ns && 1E-6 < timeIntervalNS) {
-         uint64_t total_time_diff_ns = total_current_time_ns - total_existing_time_ns;
-         proc->super.percent_cpu = ((double)total_time_diff_ns / timeIntervalNS) * 100.0;
-      } else {
-         proc->super.percent_cpu = 0.0;
-      }
-      Process_updateCPUFieldWidths(proc->super.percent_cpu);
-
-      proc->super.time = nanosecondsToCentiseconds(total_current_time_ns);
-      proc->super.nlwp = pti.pti_threadnum;
-      proc->super.m_virt = pti.pti_virtual_size / ONE_K;
-      proc->super.m_resident = pti.pti_resident_size / ONE_K;
-      proc->super.majflt = pti.pti_faults;
-      proc->super.percent_mem = (double)pti.pti_resident_size * 100.0
-                              / (double)dpl->host_info.max_mem;
-
-      proc->stime = system_time_ns;
-      proc->utime = user_time_ns;
-
-      dpl->super.kernelThreads += 0; /*pti.pti_threads_system;*/
-      dpl->super.userlandThreads += pti.pti_threadnum; /*pti.pti_threads_user;*/
-      dpl->super.totalTasks += pti.pti_threadnum;
-      dpl->super.runningTasks += pti.pti_numrunning;
+   if (PROC_PIDTASKINFO_SIZE != proc_pidinfo(Process_getPid(&proc->super), PROC_PIDTASKINFO, 0, &pti, PROC_PIDTASKINFO_SIZE)) {
+      proc->taskAccess = false;
+      return;
    }
+
+   const DarwinMachine* dhost = (const DarwinMachine*) proc->super.super.host;
+
+   uint64_t total_existing_time_ns = proc->stime + proc->utime;
+   uint64_t user_time_ns = pti.pti_total_user;
+   uint64_t system_time_ns = pti.pti_total_system;
+   uint64_t total_current_time_ns = user_time_ns + system_time_ns;
+
+   if (total_existing_time_ns < total_current_time_ns) {
+      const uint64_t total_time_diff_ns = total_current_time_ns - total_existing_time_ns;
+      proc->super.percent_cpu = ((double)total_time_diff_ns / timeIntervalNS) * 100.0;
+   } else {
+      proc->super.percent_cpu = 0.0;
+   }
+   Process_updateCPUFieldWidths(proc->super.percent_cpu);
+
+   proc->super.state = pti.pti_numrunning > 0 ? RUNNING : SLEEPING;
+   // Convert from nanoseconds to hundredths of seconds
+   proc->super.time = total_current_time_ns / 10000000ULL;
+   proc->super.nlwp = pti.pti_threadnum;
+   proc->super.m_virt = pti.pti_virtual_size / ONE_K;
+   proc->super.m_resident = pti.pti_resident_size / ONE_K;
+   proc->super.majflt = pti.pti_faults;
+   proc->super.percent_mem = (double)pti.pti_resident_size * 100.0 / (double)dhost->host_info.max_mem;
+
+   proc->stime = system_time_ns;
+   proc->utime = user_time_ns;
+
+   dpt->super.kernelThreads += 0; /*pti.pti_threads_system;*/
+   dpt->super.userlandThreads += pti.pti_threadnum; /*pti.pti_threads_user;*/
+   dpt->super.totalTasks += pti.pti_threadnum;
+   dpt->super.runningTasks += pti.pti_numrunning;
 }
 
 /*
@@ -402,7 +405,7 @@ void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessList*
  * Based on: http://stackoverflow.com/questions/6788274/ios-mac-cpu-usage-for-thread
  * and       https://github.com/max-horvath/htop-osx/blob/e86692e869e30b0bc7264b3675d2a4014866ef46/ProcessList.c
  */
-void DarwinProcess_scanThreads(DarwinProcess* dp) {
+void DarwinProcess_scanThreads(DarwinProcess* dp, DarwinProcessTable* dpt) {
    Process* proc = (Process*) dp;
    kern_return_t ret;
 
@@ -414,65 +417,106 @@ void DarwinProcess_scanThreads(DarwinProcess* dp) {
       return;
    }
 
-   task_t port;
-   ret = task_for_pid(mach_task_self(), proc->pid, &port);
-   if (ret != KERN_SUCCESS) {
-      dp->taskAccess = false;
-      return;
-   }
+   pid_t pid = Process_getPid(proc);
 
-   task_info_data_t tinfo;
-   mach_msg_type_number_t task_info_count = TASK_INFO_MAX;
-   ret = task_info(port, TASK_BASIC_INFO, (task_info_t) tinfo, &task_info_count);
+   task_t task;
+   ret = task_for_pid(mach_task_self(), pid, &task);
    if (ret != KERN_SUCCESS) {
+      // TODO: workaround for modern MacOS limits on task_for_pid()
+      if (ret != KERN_FAILURE)
+         CRT_debug("task_for_pid(%d) failed: %s", pid, mach_error_string(ret));
       dp->taskAccess = false;
       return;
    }
 
    thread_array_t thread_list;
    mach_msg_type_number_t thread_count;
-   ret = task_threads(port, &thread_list, &thread_count);
+   ret = task_threads(task, &thread_list, &thread_count);
    if (ret != KERN_SUCCESS) {
+      CRT_debug("task_threads(%d) failed: %s", pid, mach_error_string(ret));
       dp->taskAccess = false;
-      mach_port_deallocate(mach_task_self(), port);
+      mach_port_deallocate(mach_task_self(), task);
       return;
    }
 
-   integer_t run_state = 999;
-   for (unsigned int i = 0; i < thread_count; i++) {
-      thread_info_data_t thinfo;
-      mach_msg_type_number_t thread_info_count = THREAD_BASIC_INFO_COUNT;
-      ret = thread_info(thread_list[i], THREAD_BASIC_INFO, (thread_info_t)thinfo, &thread_info_count);
-      if (ret == KERN_SUCCESS) {
-         thread_basic_info_t basic_info_th = (thread_basic_info_t) thinfo;
-         if (basic_info_th->run_state < run_state) {
-            run_state = basic_info_th->run_state;
-         }
-         mach_port_deallocate(mach_task_self(), thread_list[i]);
-      }
-   }
-   vm_deallocate(mach_task_self(), (vm_address_t) thread_list, sizeof(thread_port_array_t) * thread_count);
-   mach_port_deallocate(mach_task_self(), port);
+   const bool hideUserlandThreads = dpt->super.super.host->settings->hideUserlandThreads;
 
-   /* Taken from: https://github.com/apple/darwin-xnu/blob/2ff845c2e033bd0ff64b5b6aa6063a1f8f65aa32/osfmk/mach/thread_info.h#L129 */
-   switch (run_state) {
-      case TH_STATE_RUNNING: proc->state = RUNNING; break;
-      case TH_STATE_STOPPED: proc->state = STOPPED; break;
-      case TH_STATE_WAITING: proc->state = WAITING; break;
-      case TH_STATE_UNINTERRUPTIBLE: proc->state = UNINTERRUPTIBLE_WAIT; break;
-      case TH_STATE_HALTED: proc->state = BLOCKED; break;
-      default: proc->state = UNKNOWN;
+   for (mach_msg_type_number_t i = 0; i < thread_count; i++) {
+
+      thread_identifier_info_data_t identifer_info;
+      mach_msg_type_number_t identifer_info_count = THREAD_IDENTIFIER_INFO_COUNT;
+      ret = thread_info(thread_list[i], THREAD_IDENTIFIER_INFO, (thread_info_t) &identifer_info, &identifer_info_count);
+      if (ret != KERN_SUCCESS) {
+         CRT_debug("thread_info(%d:%d) for identifier failed: %s", pid, i, mach_error_string(ret));
+         continue;
+      }
+
+      uint64_t tid = identifer_info.thread_id;
+
+      bool preExisting;
+      Process *tprocess = ProcessTable_getProcess(&dpt->super, tid, &preExisting, DarwinProcess_new);
+      tprocess->super.updated = true;
+      dpt->super.totalTasks++;
+
+      if (hideUserlandThreads) {
+         tprocess->super.show = false;
+         continue;
+      }
+
+      pid_t tprocessPid = Process_getPid(tprocess);
+      assert(tprocessPid >= 0);
+      assert((uint64_t)tprocessPid == tid);
+      (void)tprocessPid;
+
+      Process_setParent(tprocess, pid);
+      Process_setThreadGroup(tprocess, pid);
+      tprocess->super.show       = true;
+      tprocess->isUserlandThread = true;
+      tprocess->st_uid           = proc->st_uid;
+      tprocess->user             = proc->user;
+
+      thread_extended_info_data_t extended_info;
+      mach_msg_type_number_t extended_info_count = THREAD_EXTENDED_INFO_COUNT;
+      ret = thread_info(thread_list[i], THREAD_EXTENDED_INFO, (thread_info_t) &extended_info, &extended_info_count);
+      if (ret != KERN_SUCCESS) {
+         CRT_debug("thread_info(%d:%d) for extended failed: %s", pid, i, mach_error_string(ret));
+         continue;
+      }
+
+      DarwinProcess* tdproc     = (DarwinProcess*)tprocess;
+      tdproc->super.percent_cpu = extended_info.pth_cpu_usage / 10.0;
+      tdproc->stime             = extended_info.pth_system_time;
+      tdproc->utime             = extended_info.pth_user_time;
+      tdproc->super.time        = (extended_info.pth_system_time + extended_info.pth_user_time) / 10000000;
+      tdproc->super.priority    = extended_info.pth_curpri;
+
+      // TODO: depend on setting
+      const char* name = extended_info.pth_name[0] != '\0' ? extended_info.pth_name : proc->procComm;
+      Process_updateCmdline(tprocess, name, 0, strlen(name));
+
+      if (!preExisting)
+         ProcessTable_add(&dpt->super, tprocess);
    }
+
+   vm_deallocate(mach_task_self(), (vm_address_t) thread_list, sizeof(thread_port_array_t) * thread_count);
+   mach_port_deallocate(mach_task_self(), task);
 }
 
 
 const ProcessClass DarwinProcess_class = {
    .super = {
-      .extends = Class(Process),
-      .display = Process_display,
-      .delete = Process_delete,
-      .compare = Process_compare
+      .super = {
+         .extends = Class(Process),
+         .display = Row_display,
+         .delete = Process_delete,
+         .compare = Process_compare
+      },
+      .isHighlighted = Process_rowIsHighlighted,
+      .isVisible = Process_rowIsVisible,
+      .matchesFilter = Process_rowMatchesFilter,
+      .compareByParent = Process_compareByParent,
+      .sortKeyString = Process_rowGetSortKey,
+      .writeField = DarwinProcess_rowWriteField
    },
-   .writeField = DarwinProcess_writeField,
-   .compareByKey = DarwinProcess_compareByKey,
+   .compareByKey = DarwinProcess_compareByKey
 };
