@@ -10,9 +10,13 @@ in the source distribution for its full text.
 
 #include "dragonflybsd/Platform.h"
 
+#include <devstat.h>
+#include <errno.h>
+#include <ifaddrs.h>
 #include <math.h>
 #include <time.h>
 #include <sys/resource.h>
+#include <sys/socket.h>
 #include <sys/sysctl.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -34,6 +38,7 @@ in the source distribution for its full text.
 #include "TasksMeter.h"
 #include "UptimeMeter.h"
 #include "XUtils.h"
+#include "dragonflybsd/DragonFlyBSDMachine.h"
 #include "dragonflybsd/DragonFlyBSDProcess.h"
 #include "dragonflybsd/DragonFlyBSDProcessTable.h"
 #include "generic/fdstat_sysctl.h"
@@ -115,6 +120,8 @@ const MeterClass* const Platform_meterTypes[] = {
    &RightCPUs4Meter_class,
    &LeftCPUs8Meter_class,
    &RightCPUs8Meter_class,
+   &DiskIOMeter_class,
+   &NetworkIOMeter_class,
    &FileDescriptorMeter_class,
    &BlankMeter_class,
    NULL
@@ -193,7 +200,7 @@ double Platform_setCPUValues(Meter* this, unsigned int cpu) {
 
    v[CPU_METER_NICE]   = cpuData->nicePercent;
    v[CPU_METER_NORMAL] = cpuData->userPercent;
-   if (super->settings->detailedCPUTime) {
+   if (host->settings->detailedCPUTime) {
       v[CPU_METER_KERNEL]  = cpuData->systemPercent;
       v[CPU_METER_IRQ]     = cpuData->irqPercent;
       this->curItems = 4;
@@ -247,15 +254,90 @@ void Platform_getFileDescriptors(double* used, double* max) {
 }
 
 bool Platform_getDiskIO(DiskIOData* data) {
-   // TODO
-   (void)data;
-   return false;
+   struct statinfo dev_stats = { 0 };
+   struct device_selection* dev_sel = NULL;
+   int n_selected, n_selections;
+   long sel_gen;
+
+   dev_stats.dinfo = xCalloc(1, sizeof(struct devinfo));
+
+   int ret = getdevs(&dev_stats);
+   if (ret < 0) {
+      CRT_debug("getdevs() failed [%d]: %s", ret, strerror(errno));
+      free(dev_stats.dinfo);
+      return false;
+   }
+
+   ret = selectdevs(&dev_sel, &n_selected, &n_selections, &sel_gen,
+         dev_stats.dinfo->generation, dev_stats.dinfo->devices, dev_stats.dinfo->numdevs,
+         NULL, 0, NULL, 0, DS_SELECT_ONLY, dev_stats.dinfo->numdevs, 1);
+   if (ret < 0) {
+      CRT_debug("selectdevs() failed [%d]: %s", ret, strerror(errno));
+      free(dev_stats.dinfo);
+      return false;
+   }
+
+   uint64_t bytesReadSum = 0;
+   uint64_t bytesWriteSum = 0;
+   uint64_t busyMsTimeSum = 0;
+   uint64_t numDisks = 0;
+
+   for (int i = 0; i < dev_stats.dinfo->numdevs; i++) {
+      const struct devstat* device = &dev_stats.dinfo->devices[dev_sel[i].position];
+
+      switch (device->device_type & DEVSTAT_TYPE_MASK) {
+      case DEVSTAT_TYPE_DIRECT:
+      case DEVSTAT_TYPE_SEQUENTIAL:
+      case DEVSTAT_TYPE_WORM:
+      case DEVSTAT_TYPE_CDROM:
+      case DEVSTAT_TYPE_OPTICAL:
+      case DEVSTAT_TYPE_CHANGER:
+      case DEVSTAT_TYPE_STORARRAY:
+      case DEVSTAT_TYPE_FLOPPY:
+         break;
+      default:
+         continue;
+      }
+
+      bytesReadSum  += device->bytes_read;
+      bytesWriteSum += device->bytes_written;
+      busyMsTimeSum += (device->busy_time.tv_sec * 1000 + device->busy_time.tv_usec / 1000);
+      numDisks++;
+   }
+
+   data->totalBytesRead = bytesReadSum;
+   data->totalBytesWritten = bytesWriteSum;
+   data->totalMsTimeSpend = busyMsTimeSum;
+   data->numDisks = numDisks;
+
+   free(dev_stats.dinfo);
+   return true;
 }
 
 bool Platform_getNetworkIO(NetworkIOData* data) {
-   // TODO
-   (void)data;
-   return false;
+   struct ifaddrs* ifaddrs = NULL;
+
+   if (getifaddrs(&ifaddrs) != 0)
+      return false;
+
+   for (const struct ifaddrs* ifa = ifaddrs; ifa; ifa = ifa->ifa_next) {
+      if (!ifa->ifa_addr)
+         continue;
+      if (ifa->ifa_addr->sa_family != AF_LINK)
+         continue;
+      if (ifa->ifa_flags & IFF_LOOPBACK)
+         continue;
+
+      const struct if_data* ifd = (const struct if_data*)ifa->ifa_data;
+
+      data->bytesReceived += ifd->ifi_ibytes;
+      data->packetsReceived += ifd->ifi_ipackets;
+      data->bytesTransmitted += ifd->ifi_obytes;
+      data->packetsTransmitted += ifd->ifi_opackets;
+   }
+
+   freeifaddrs(ifaddrs);
+   return true;
 }
 
 void Platform_getBattery(double* percent, ACPresence* isOnAC) {
