@@ -3,14 +3,16 @@
 /*
 htop - dragonflybsd/ProcessField.h
 (C) 2020 htop dev team
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
 
 #define PLATFORM_PROCESS_FIELDS  \
-   JID   = 100,                  \
-   JAIL  = 101,                  \
+   JID = 100,                    \
+   JAIL = 101,                   \
+                                 \
+   DUMMY_BUMP_FIELD = CWD,       \
    // End of list
 
 
