@@ -13,12 +13,23 @@ in the source distribution for its full text.
 #include "Process.h"
 #include "Machine.h"
 
+typedef enum {
+   SCHEDCLASS_UNKNOWN = 0,
+
+   SCHEDCLASS_INTR_THREAD, /* interrupt thread */
+   SCHEDCLASS_REALTIME,
+   SCHEDCLASS_TIMESHARE, /* Regular scheduling */
+   SCHEDCLASS_IDLE,
+
+   MAX_SCHEDCLASS,
+} FreeBSDSchedClass;
 
 typedef struct FreeBSDProcess_ {
    Process super;
    int   jid;
    char* jname;
    char* emul;
+   FreeBSDSchedClass sched_class;
 } FreeBSDProcess;
 
 extern const ProcessClass FreeBSDProcess_class;
